<?php

namespace Drupal\calculation_fields\Element;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\Number;
use Drupal\webform\Utility\WebformElementHelper;
use Symfony\Component\ExpressionLanguage\ExpressionLanguage;

/**
 * Provides a custom element capable to realize some math expression displaying,
 * the result of the expression.
 *
 * @FormElement("form_element_math")
 */
class FormMathElement extends Number {

  /**
   * {@inheritDoc}
   */
  public function getInfo() {
    $info = parent::getInfo();
    $info['#evaluation_fields'] = NULL;
    $info['#process'] = [self::class . '::processFormMathElement'];
    $info['#element_validate'] = [self::class . '::validate'];
    return $info;
  }

  /**
   * Validate form math element callback.
   *
   * To avoid disruptive data from the client, every submit evaluate the
   * expression and set the correct/expected value for the element.
   */
  public static function validate(&$element, FormStateInterface $form_state, &$complete_form) {
    $expression = static::buildFieldEvaluation($element["#evaluation_fields"], $form_state);
    $value = NestedArray::getValue($form_state->getValues(), $element['#parents']);
    if (empty($expression) || empty($value)) {
      return;
    }
    $expressionLanguage = new ExpressionLanguage();
    $result = $expressionLanguage->evaluate($expression);
    if ($result !== $value) {
      $form_state->setValue($element["#name"], $result);
    }
  }

  /**
   * {@inheritDoc}
   */
  public static function preRenderNumber($element) {
    $element['#attributes']['type'] = 'number';
    $element['#attributes']['readonly'] = 'readonly';
    Element::setAttributes($element, ['id', 'name', 'value', 'step', 'min', 'max', 'placeholder', 'size', 'fields']);
    static::setAttributes($element, ['form-number', 'js-form-math-element']);
    $element['#attributes']['data-evaluate-expression'] = $element["#evaluation_fields"];
    $element['#attributes']['data-evaluate-fields'] = static::evaluateFields($element["#evaluation_fields"]);
    $element['#attached']['library'][] = 'calculation_fields/calculation_fields';
    return $element;
  }

  /**
   * Process each form math element setting custom properties.
   *
   * Set two properties:
   * - data-evaluate-context: used to gurantee that the input will use values,
   * in the correct form context.
   * - data-evaluate-deep-fields: deep fields properties is used to store
   * values of fields that are not being displayed like, like in a multistep
   * form in a webform for example.
   *
   * @param array $element
   *   The element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state instance.
   */
  public static function processFormMathElement(array $element, FormStateInterface $form_state, $form) {
    $element['#attributes']['data-evaluate-context'] = str_replace('_', '-', $form["#form_id"]);
    if (isset($element['#evaluation_round'])) {
      $element['#attributes']['data-evaluate-round'] = (int) $element['#evaluation_round'];
    }
    $dependency_fields = static::evaluateFields($element["#evaluation_fields"]);

    $has_multiple_steps = $form["progress"]["#current_page"] ?? NULL;
    $completed_form = $form_state->getCompleteForm();
    $trigerred = $form_state->getTriggeringElement();

    $values_dependencies = [];
    foreach ($dependency_fields as $field) {
      if (!empty($form['elements'])) {
        $d_element = WebformElementHelper::getElement($form['elements'], $field);
      }
      else {
        $d_element = WebformElementHelper::getElement($form, $field);
      }

      if ($d_element["#webform_parent_key"] !== $has_multiple_steps || !$d_element['#access']) {
        $values_dependencies[$field] = $form_state->getValue($field);
      }
    }
    $element['#attributes']['data-evaluate-deep-fields'] = json_encode($values_dependencies);
    return $element;
  }

  /**
   * Build expression field replacing field name for this value.
   *
   * Example: (:field_name_1 + :field_name2) became (2 + 2).
   *
   * @param string $evaluation
   *   The field expression.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return string
   *   The expression with values.
   */
  public static function buildFieldEvaluation($evaluation, FormStateInterface $form_state) {
    $fields = static::evaluateFields($evaluation);
    $collected_values = [];
    foreach ($fields as $field) {
      $value = $form_state->getValue($field);
      if (!empty($value)) {
        $collected_values[$field] = $value;
      }
    }
    if (count($collected_values) !== count(array_unique($fields))) {
      return null;
    }
    $exp = $evaluation;
    foreach ($collected_values as $field => $value) {
      $exp = str_replace(':' . $field, $value, $exp);
    }
    return $exp;
  }

  /**
   * Evaluation fields expression getting the field names.
   *
   * @param string $evaluation_fields
   *   Evaluation field expression.
   *
   * @return array
   *   An array with the field names.
   */
  public static function evaluateFields($evaluation_fields): array {
    $expression_fields = explode(':', $evaluation_fields) ?? [];
    $expression_fields = array_filter($expression_fields);
    $fields = [];
    foreach ($expression_fields as $field) {
      $items = explode(' ', $field);
      $items = array_map(function ($item) {
        return preg_replace("/[^a-zA-Z0-9_]/", "", $item);
      }, $items);
      $filteredItems = array_filter($items, function ($item) {
        return !is_numeric($item) && !empty($item);
      });
      $fields = array_merge($fields, $filteredItems);
    }
    return $fields;
  }


}
